<!DOCTYPE html>
<html lang="en">
    <head>
        <!-- Required meta tags always come first -->
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

        <!-- Bootstrap CSS -->
        <link rel="stylesheet" href="/static/css/bootstrap.min.css">
        <link rel="stylesheet" href="/static/css/bootstrap-icons.min.css">
        <link rel="stylesheet" href="/static/css/custom.css">

        <title>{{ website.html_title }} | Brother QL</title>
    </head>
    <body>
        <nav class="navbar navbar-expand-lg navbar-dark bg-dark">
            <div class="container">
                <div class="navbar-header">
                    <button type="button" class="navbar-toggler collapsed" data-bs-toggle="collapse" data-bs-target="#navbar" aria-expanded="false" aria-controls="navbar">
                        <span class="sr-only">Toggle navigation</span>
                        <span class="icon-bar"></span>
                        <span class="icon-bar"></span>
                        <span class="icon-bar"></span>
                    </button>
                    <a class="navbar-brand" href="/">{{ website.page_title }}</a>
                </div>
            </div>
        </nav>
        <div class="jumbotron">
            <div class="container">
                {% block jumbotron %}{% endblock %}
            </div>
        </div>
        <div class="container">
            {% block content %}{% endblock %}
        </div>

        <!-- Javascript -->
        <script src="/static/js/bootstrap.min.js"></script>
        <script type="text/javascript">
            {% block javascript %}{% endblock %}
        </script>
    </body>
</html>

